bc(1) -- arbitrary-precision arithmetic language and calculator
===============================================================

SYNOPSIS
--------

`bc` [`-hilqsvVw`] [`--help`] [`--interactive`] [`--mathlib`] [`--quiet`]
[`--standard`] [`--warn`] [`--version`] [`-e` *expr*] [`--expression=`*expr*...]
[`-f` *file*...] [`-file=`*file*...] [*file*...]

DESCRIPTION
-----------

bc(1) is an interactive processor for a language first standardized in 1991 by
POSIX. (The current standard is [here][1].) The language provides unlimited
precision decimal arithmetic and is somewhat C-like, but there are differences.
Such differences will be noted in this document.

After parsing and handling options, this bc(1) reads any files given on the
command line and executes them before reading from `stdin`.

With all build options but extra math enabled this bc(1) is a drop-in
replacement for ***any*** bc(1), including (and especially) the GNU bc(1).

OPTIONS
-------

The following are the options that bc(1) accepts.

  * `-h`, `--help`:
    Prints a usage message and quits.

  * `-i`, `--interactive`:
    Forces interactive mode.

    Per the [standard][1], bc(1) has an interactive mode and a non-interactive
    mode. The interactive mode is turned on automatically when both `stdin` and
    `stdout` are hooked to a terminal, but this flag can turn it on in other
    cases. In interactive mode, bc(1) attempts to recover from errors and
    flushes `stdout` as soon as execution is done for the current input.

  * `-l`, `--mathlib`:
    Sets `scale` (see the Scale section) to `20` and loads the included math
    library before running any code, including any expressions or files specified
    on the command line.

    To learn what is in the library, see the [Library](#LIBRARY) section.

  * `-q`, `--quiet`:
    Do not print copyright header. bc(1) will also suppress the header in
    non-interactive mode.

    This is mostly for compatibility with the [GNU bc(1)][2].

  * `-s`, `--standard`:
    Process exactly the language defined by the [standard][1] and error if any
    extensions are used.

  * `-v`, `-V`, `--version`:
    Print the version information (copyright header) and exit.

  * `-w`, `--warn`:
    Like `-s` and `--standard`, except that warnings (and not errors) are given
    for non-standard extensions.

  * `-e` *expr*, `--expression`=*expr*:
    Evaluates `expr`. If multiple expressions are given, they are evaluated in
    order. If files are given as well (see below), the expressions and files are
    evaluated in the order given. This means that if a file is given before an
    expression, the file is read in and evaluated first.

  * `-f` *file*, `--file`=*file*:
    Reads in `file` and evaluates it. If expressions are also given (see above),
    the expressions are evaluated in the order given.

SYNTAX
------

The syntax for bc(1) programs is mostly C-like, with some differences. This
bc(1) follows the [POSIX standard][1], which is a much more thorough resource
for the language this bc(1) accepts. This section is meant to be a summary and a
listing of all the extensions to the [standard][1].

In the sections below, `E` means expression, `S` means statement, and `I` means
identifier

Identifiers (`I`) start with a lowercase letter and can be followed by any
number (up to `BC_NAME_MAX-1`) of lowercase letters (`a-z`), digits (`0-9`), and
underscores (`_`). The regex is `[a-z][a-z0-9_]*` Identifiers with more than one
character (letter) are an extension.

`ibase` is a global variable determining how to interpret constant numbers. It
is the "input" base, or the number base used for interpreting input numbers.
`ibase` is initially `10`. If the `-s` (`--standard`) and `-w` (`--warn`) flags
were not given on the command line, the max allowable value for `ibase` is `36`.
Otherwise, it is `16`. The min allowable value for `ibase` is `2`.

`obase` is a global variable determining how to output results. It is the
"output" base, or the number base used for outputting numbers. `obase` is
initially `10`. The max allowable value for `obase` is `BC_BASE_MAX`. The min
allowable value for `obase` is `2`.

The **scale** of an expression is the number of digits in the result of the
expression right of the decimal point, and `scale` is a global variable that
sets the precision of any operations, with exceptions. `scale` is initially `0`.
`scale` cannot be negative.

bc(1) has both **global** variables and **local** variables. All **local**
variables are local to the function; they are parameters or are introduced in a
function's `auto` list (see [Functions](#FUNCTIONS)). If a variable is
accessed which is not a parameter or in the `auto` list, it is assumed to be
**global**. If a parent function has a **local** variable version of a
**global** variable that is accessed by a function that it calls, the value of
that **global** variable in the child function is the value of the variable in
the parent function, not the value of the actual **global** variable.

All of the above applies to arrays as well.

The value of a statement that is an expression (i.e., any of the
[Named Expressions](#bc-named-expressions) or [Operands](#bc-operands)) is
printed unless the lowest precedence operator is an
[`assignment`](#bc-assignment) operator ***and*** the expression is not
surrounded by parentheses. The value that is printed is also assigned to the
special variable `last`. This is a non-portable extension. A single dot may also
be used as a synonym for `last`.

Either semicolons or newlines may separate statements.

### Comments

There are two kinds of comments:

1.	Block comments are enclosed in `/*` and `*/`.
2.	Line comments go from `#` until, and not including, the next newline. This
	is a non-portable extension.

<a name="bc-named-expressions"/>

### Named Expressions

1.	Variables: `I`
2.	Array Elements: `I[E]`
3.	`ibase`
4.	`obase`
5.	`scale`
6.	`last` or a single dot (`.`)

Variables and arrays do not interfere; users can have arrays named the same as
variables. This also applies to [functions](#FUNCTIONS), so a user can have
a variable, array, and function that all have the same name.

Named expressions are required as the operand of
[`increment`/`decrement` operators](#bc-increment-decrement) and as the left
side of [`assignment` operators](#bc-assignment).

<a name="bc-operands"/>

### Operands

1.	Numbers (see [Numbers](#bc-numbers) below).
2.	`(E)`: The value of `E` (used to change precedence).
3.	`sqrt(E)`: The square root of `E`.
4.	`length(E)`: The number of significant decimal digits in `E`.
5.	`length(I[])`: The number of elements in the array `I`. This is a
	non-portable extension.
6.	`scale(E)`: `E`'s **scale**.
7.	`abs(E)`: The absolute value of `E`. This is a non-portable extension.

<a name="bc-numbers"/>

### Numbers

Numbers are strings made up of digits, uppercase letters, and at most `1` period
for a radix. Numbers can have up to `BC_NUM_MAX` digits. Uppercase letters
equal `9` + their position in the alphabet (i.e., `A` equals `10`, or `9 + 1`).
If a digit or letter makes no sense with the current value of `ibase`, they are
set to the value of the highest valid digit in `ibase`.

Single-character numbers (i.e., `A`) take the value that they would have if they
were valid digits, regardless of the value of `ibase`. This means that `A`
always equals decimal `10` and `Z` always equals decimal `35`.

### Operators

The following arithmetic and logical operators can be used. They are listed in
order of decreasing precedence. Operators in the same group have the same
precedence.

  * `++` `--`:
    Type: Prefix and Postfix

    Associativity: None

    Description: `increment`, `decrement`

  * `-` `!`:
    Type: Prefix

    Associativity: None

    Description: `negation`, `boolean not`

  * `$`:
    Type: Postfix

    Associativity: None

    Description: `truncation`

  * `@`:
    Type: Binary

    Associativity: Right

    Description: `set precision`

  * `^`:
    Type: Binary

    Associativity: Right

    Description: `power`

  * `*` `/` `%`:
    Type: Binary

    Associativity: Left

    Description: `multiply`, `divide`, `modulus`

  * `+` `-`:
    Type: Binary

    Associativity: Left

    Description: `add`, `subtract`

  * `<<` `>>`:
    Type: Binary

    Associativity: Left

    Description: `shift left`, `shift right`

  * `=` `<<=` `>>=` `+=` `-=` `*=` `/=` `%=` `^=` `@=`:
    Type: Binary

    Associativity: Right

    Description: `assignment`

  * `==` `<=` `>=` `!=` `<` `>`:
    Type: Binary

    Associativity: Left

    Description: `relational`

  * `&&`:
    Type: Binary

    Associativity: Left

    Description: `boolean and`

  * `||`:
    Type: Binary

    Associativity: Left

    Description: `boolean or`

The operators will be described in more detail below.

<a name="bc-increment-decrement"/>

  * `++` `--`:
    The prefix and postfix `increment` and `decrement` operators behave exactly
    like they would in C. They require a [named expression](#named-expressions)
    as an operand.

  * `-`:
    The `negation` operator returns `0` if a user attempts to negate any
    expression with the value `0`. Otherwise, a copy of the expression with its
    sign flipped is returned.

  * `!`:
    The `boolean not` operator returns `1` if the expression is `0`, or `0`
    otherwise.

    This is a non-portable extension.

  * `$`:
    The `truncation` operator returns a copy of the given expression with all of
    its **scale** removed.

    This is a non-portable extension.

    This is only available if bc(1) has been compiled with the extra math option
    enabled.

  * `@`:
    The `set precision` operator takes two expressions and returns a copy of the
    first with its **scale** equal to the value of the second expression. That
    could either mean that the number is returned without change (if the first
    expression's **scale** matches the value of the second expression), extended
    (if it is less), or truncated (if it is more).

    The second expression must be an integer (no **scale**) and non-negative.

    This is a non-portable extension.

    This is only available if bc(1) has been compiled with the extra math option
    enabled.

  * `^`:
    The `power` operator (not the `exclusive or` operator, as it would be in C)
    takes two expressions and raises the first to the power of the value of the
    second.

    The second expression must be an integer (no **scale**).

  * `*`:
    The `multiply` operator takes two expressions, multiplies them, and returns
    the product. If `a` is the **scale** of the first expression and `b` is the
    **scale** of the second expression, the scale of the result is equal to
    `min(a+b,max(scale,a,b))` where `min` and `max` return the obvious values.

  * `/`:
    The `divide` operator takes two expressions, divides them, and returns the
    quotient. The scale of the result shall be the value of `scale`.

  * `%`:
    The `modulus` operator takes two expressions, `a` and `b`, and evaluates
    them by 1) Computing `a/b` to current `scale` and 2) Using the result of
    step 1 to calculate `a-(a/b)*b` to scale `max(scale+scale(b),scale(a))`.

  * `+`:
    The `add` operator takes two expressions, `a` and `b`, and returns the sum,
    with a **scale** equal to the max of the **scale**s of `a` and `b`.

  * `-`:
    The `subtract` operator takes two expressions, `a` and `b`, and returns the
    difference, with a **scale** equal to the max of the **scale**s of `a` and
    `b`.

  * `<<`:
    The `left shift` operator takes two expressions, `a` and `b`, and returns a
    copy of the value of `a` with its decimal point moved `b` places to the
    right.

    The second expression must be an integer (no **scale**) and non-negative.

    This is a non-portable extension.

    This is only available if bc(1) has been compiled with the extra math option
    enabled.

  * `>>`:
    The `right shift` operator takes two expressions, `a` and `b`, and returns a
    copy of the value of `a` with its decimal point moved `b` places to the
    left.

    The second expression must be an integer (no **scale**) and non-negative.

    This is a non-portable extension.

    This is only available if bc(1) has been compiled with the extra math option
    enabled.

<a name="bc-assignment"/>

  * `=` `<<=` `>>=` `+=` `-=` `*=` `/=` `%=` `^=` `@=`:
    The `assignment` operators take two expressions, `a` and `b` where `a` is a
    [named expression](#bc-named-expressions).

    For `=`, `b` is copied and the result is assigned to `a`. For all others,
    `a` and `b` are applied as operands to the corresponding arithmetic
    operator and the result is assigned to `a`.

    The `assignment` operators that correspond to operators that are extensions
    are themselves extensions.

    Also, those `assignment` operators that are extensions are only available if
    bc(1) has been compiled with the extra math option enabled.

  * `==` `<=` `>=` `!=` `<` `>`:
    The `relational` operators compare two expressions, `a` and `b`, and if the
    relation holds, according to C language semantics, the result is `1`.
    Otherwise, it is `0`.

    Note that unlike in C, these operators have a lower precedence than the
    `assignment` operators, which means that `a=b>c` is interpreted as
    `(a=b)>c`.

    Also, unlike the [standard][1] requires, these operators can appear anywhere
    any other expressions can be used. This allowance is an extension.

  * `&&`:
    The `boolean and` operator takes two expressions and returns `1` if both
    expressions are non-zero, `0` otherwise.

    This is ***not*** a short-circuit operator.

    This is a non-portable extension.

  * `||`:
    The `boolean or` operator takes two expressions and returns `1` if one of
    the expressions is non-zero, `0` otherwise.

    This is ***not*** a short-circuit operator.

    This is a non-portable extension.

### Statements

The following items are statements:

1.	`E`
2.	`{` `S` `;` ... `;` `S` `}`
3.	`if` `(` `E` `)` `S`
4.	`if` `(` `E` `)` `S` `else` `S`
5.	`while` `(` `E` `)` `S`
6.	`for` `(` `E` `;` `E` `;` `E` `)` `S`
7.	An empty statement
8.	`break`
9.	`continue`
10.	`quit`
11.	`halt`
12.	`limits`
13.	A string of characters, enclosed in double quotes
14.	`print` `E` `,` ... `,` `E`

Numbers 4, 9, 11, 12, and 14 are extensions.

Also, as an extension, any or all of the expressions in the header of a for loop
may be omitted. If the condition (second expression) is omitted, it is assumed
to be a constant `1`.

The `break` statement causes a loop to stop iterating and resume execution
immediately following a loop. This is only allowed in loops.

The `continue` statement causes a loop iteration to stop early and returns to
the start of the loop, including testing the loop condition. This is only
allowed in loops.

The `if` `else` statement does the same thing as in C.

The `quit` statement causes bc(1) to quit, even if it is on a branch that will
not be executed (it is a compile-time command).

The `halt` statement causes bc(1) to quit, if it is executed. (Unlike `quit` if
it is on a branch of an `if` statement that is not executed, bc(1) does not
quit.)

The `limits` statement prints the limits that this bc(1) is subject to. This is
like the `quit` statement in that it is a compile-time command.

### Print Statement

The "expressions" in a `print` statement may also be strings. If they are, there
are backslash escape sequences that are interpreted specially. What those
sequences are, and what they cause to be printed, are shown below:

  * `\a`:
    `\a`

  * `\b`:
    `\b`

  * `\\`:
    `\`

  * `\e`:
    `\`

  * `\f`:
    `\f`

  * `\n`:
    `\n`

  * `\q`:
    `"`

  * `\r`:
    `\r`

  * `\t`:
    `\t`

Any other character following a backslash causes the backslash and character to
be printed as-is.

Any non-string expression in a print statement shall be assigned to `last`, like
any other expression that is printed.

FUNCTIONS
---------

Function definitions follow what is required by the bc spec:

```
define I(I,...,I){
	auto I,...,I
	S;...;S
	return(E)
}
```

Any `I` in the parameter list or `auto` list may be replaced with `I[]` to make
a parameter or `auto` var an array.

As a non-portable extension, the opening brace of a `define` statement may
appear on the next line.

The return statement may also be in the following forms:

1.	`return`
2.	`return` `(` `)`
3.	`return` `E`

The first two, or not specifying a `return` statement, is equivalent to
`return (0)`.

<a name="void-functions"/>

### Void Functions

Functions can also be void functions, defined as follows:

```
define void I(I,...,I){
	auto I,...,I
	S;...;S
	return(E)
}
```

They can only be used as standalone expressions, where such an expression would
be printed alone, except in a print statement.

Void functions can only use the first two `return` statements listed above.

The word `void` is not treated as a keyword; it is still possible to have
variables, arrays, and functions named `void`.

This is a non-portable extension.

### Array References

For any array in the parameter list, if the array is declared in the form

```
*I[]
```

it is a **reference**. Any changes to the array in the function are reflected
when the function returns to the array that was passed in.

Other than this, all function arguments are passed by value.

This is a non-portable extension.

This is only available if bc(1) has been compiled with the array references
option enabled.

LIBRARY
-------

All of the functions below, including the functions in the
[extended library](#extended-library) if bc(1) has been compiled with the extra
math option enabled, are available when the `-l` or `--mathlib` command-line
flags are given.

<a name="standard-library"/>

### Standard Library

The [standard][1] defines the following functions for the math library:

  * `s(x)`:
    Returns the sine of `x`, which is assumed to be in radians.

    This is a [transcendental function][5].

  * `c(x)`:
    Returns the cosine of `x`, which is assumed to be in radians.

    This is a [transcendental function][5].

  * `a(x)`:
    Returns the arctangent of `x`, in radians.

    This is a [transcendental function][5].

  * `l(x)`:
    Returns the natural logarithm of `x`.

    This is a [transcendental function][5].

  * `e(x)`:
    Returns the mathematical constant `e` raised to the power of `x`.

    This is a [transcendental function][5].

  * `j(x, n)`:
    Returns the bessel integer order `n` (truncated) of `x`.

    This is a [transcendental function][5].

<a name="extended-library"/>

### Extended Library

In addition to the [standard library](#standard-library), if bc(1) has been
built with the extra math option, the following functions are available when
either the `-l` or `--mathlib` options are given.

However, the extended library is ***not*** loaded when the `-s`/`--standard` or
`-w`/`--warn` options are given since they are not part of the library defined
by the [standard][1].

  * `r(x, p)`:
    Returns `x` rounded to `p` decimal places according to the rounding mode
    [round half away from `0`][3].

  * `f(x)`:
    Returns the factorial of the truncated absolute value of `x`.

  * `perm(n, k)`:
    Returns the permutation of the truncated absolute value of `n` of the
    truncated absolute value of `k`, if `k <= n`. If not, it returns `0`.

  * `comb(n, k)`:
    Returns the combination of the truncated absolute value of `n` of the
    truncated absolute value of `k`, if `k <= n`. If not, it returns `0`.

  * `l2(x)`:
    Returns the logarithm base `2` of `x`.

    This is a [transcendental function][5].

  * `l10(x)`:
    Returns the logarithm base `10` of `x`.

    This is a [transcendental function][5].

  * `log(x, b)`:
    Returns the logarithm base `b` of `x`.

    This is a [transcendental function][5].

  * `pi(p)`:
    Returns `pi` to `p` decimal places.

    This is a [transcendental function][5].

  * `ubytes(x)`:
    Returns the numbers of unsigned integer bytes required to hold the truncated
    absolute value of `x`.

  * `sbytes(x)`:
    Returns the numbers of signed, two's-complement integer bytes required to
    hold the truncated value of `x`.

  * `hex(x)`:
    Outputs the hexadecimal (base `16`) representation of `x`.

    This is a [void](#void-functions) function.

  * `binary(x)`:
    Outputs the binary (base `2`) representation of `x`.

    This is a [void](#void-functions) function.

  * `output(x, b)`:
    Outputs the base `b` representation of `x`.

    This is a [void](#void-functions) function.

  * `uint(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as an unsigned
    integer in as few power of two bytes as possible. Both outputs are split
    into bytes separated by spaces.

    If `x` is not an integer or is negative, an error message is printed
    instead.

    This is a [void](#void-functions) function.

  * `int(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as a signed,
    two's-complement integer in as few power of two bytes as possible. Both
    outputs are split into bytes separated by spaces.

    If `x` is not an integer, an error message is printed instead.

    This is a [void](#void-functions) function.

  * `uintn(x, n)`:
    Outputs the representation, in binary and hexadecimal, of `x` as an unsigned
    integer in `n` bytes. Both outputs are split into bytes separated by spaces.

    If `x` is not an integer, is negative, or cannot fit into `n` bytes, an
    error message is printed instead.

    This is a [void](#void-functions) function.

  * `intn(x, n)`:
    Outputs the representation, in binary and hexadecimal, of `x` as a signed,
    two's-complement integer in `n` bytes. Both outputs are split into bytes
    separated by spaces.

    If `x` is not an integer or cannot fit into `n` bytes, an error message is
    printed instead.

    This is a [void](#void-functions) function.

  * `uint8(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as an unsigned
    integer in `1` byte. Both outputs are split into bytes separated by spaces.

    If `x` is not an integer, is negative, or cannot fit into `1` byte, an error
    message is printed instead.

    This is a [void](#void-functions) function.

  * `int8(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as a signed,
    two's-complement integer in `1` byte. Both outputs are split into bytes
    separated by spaces.

    If `x` is not an integer or cannot fit into `1` byte, an error message is
    printed instead.

    This is a [void](#void-functions) function.

  * `uint16(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as an unsigned
    integer in `2` bytes. Both outputs are split into bytes separated by spaces.

    If `x` is not an integer, is negative, or cannot fit into `2` bytes, an
    error message is printed instead.

    This is a [void](#void-functions) function.

  * `int16(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as a signed,
    two's-complement integer in `2` bytes. Both outputs are split into bytes
    separated by spaces.

    If `x` is not an integer or cannot fit into `2` bytes, an error message is
    printed instead.

    This is a [void](#void-functions) function.

  * `uint32(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as an unsigned
    integer in `4` bytes. Both outputs are split into bytes separated by spaces.

    If `x` is not an integer, is negative, or cannot fit into `4` bytes, an
    error message is printed instead.

    This is a [void](#void-functions) function.

  * `int32(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as a signed,
    two's-complement integer in `4` bytes. Both outputs are split into bytes
    separated by spaces.

    If `x` is not an integer or cannot fit into `4` bytes, an error message is
    printed instead.

    This is a [void](#void-functions) function.

  * `uint64(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as an unsigned
    integer in `8` bytes. Both outputs are split into bytes separated by spaces.

    If `x` is not an integer, is negative, or cannot fit into `8` bytes, an
    error message is printed instead.

    This is a [void](#void-functions) function.

  * `int64(x)`:
    Outputs the representation, in binary and hexadecimal, of `x` as a signed,
    two's-complement integer in `8` bytes. Both outputs are split into bytes
    separated by spaces.

    If `x` is not an integer or cannot fit into `8` bytes, an error message is
    printed instead.

    This is a [void](#void-functions) function.

  * `hex_uint(x, n)`:
    Outputs the representation of the truncated absolute value of `x` as an
    unsigned integer in hexadecimal using `n` bytes. Not all of the value will
    be output if `n` is too small.

    This is a [void](#void-functions) function.

  * `binary_uint(x, n)`:
    Outputs the representation of the truncated absolute value of `x` as an
    unsigned integer in binary using `n` bytes. Not all of the value will be
    output if `n` is too small.

    This is a [void](#void-functions) function.

  * `output_uint(x, n)`:
    Outputs the representation of the truncated absolute value of `x` as an
    unsigned integer in the current [`obase`](#obase) using `n` bytes. Not all
    of the value will be output if `n` is too small.

    This is a [void](#void-functions) function.

  * `output_byte(x, i)`:
    Outputs byte `i` of the truncated absolute value of `x`, where `0` is the
    least significant byte and `number_of_bytes - 1` is the most significant
    byte.

    This is a [void](#void-functions) function.

<a name="transcendental-functions"/>

### Transcendental Functions

All transcendental functions can return slightly inaccurate results (up to 1
[ULP][4]). This is unavoidable, and [this article][6] explains why it is
impossible and unnecessary to calculate exact results for the transcendental
functions.

Because of the possible inaccuracy, I recommend that users call those functions
with the precision (`scale`) set to at least 1 higher than is necessary. If
exact results are *absolutely* required, users can double the precision
(`scale`) and then truncate.

The transcendental functions in the standard math library are:

* `s(x)`
* `c(x)`
* `a(x)`
* `l(x)`
* `e(x)`
* `j(x, n)`

The transcendental functions in the extended math library are:

* `l2(x)`
* `l10(x)`
* `log(x, b)`
* `pi(p)`

SIGNAL HANDLING
---------------

If bc(1) has been compiled with the signal handling, sending a `SIGINT` will
cause bc(1) to stop  execution of the current input and reset, asking for more
input.

Otherwise, any signals cause bc(1) to exit.

COMMAND LINE HISTORY
--------------------

bc(1) supports interactive command-line editing, if compiled with the history
option enabled. If `stdin` is hooked to a terminal, it is enabled. Previous
lines can be recalled and edited with the arrow keys.

SEE ALSO
--------

dc(1)

STANDARDS
---------

bc(1) is compliant with the [IEEE Std 1003.1-2017 (“POSIX.1-2017”)][1]
specification. The flags `-efhiqsvVw`, all long options, and the extensions
noted above are extensions to that specification.

AUTHOR
------

This bc(1) was made from scratch by Gavin D. Howard.

BUGS
----

None are known. Report bugs at https://github.com/gavinhoward/bc.

[1]: https://pubs.opengroup.org/onlinepubs/9699919799/utilities/bc.html
[2]: https://www.gnu.org/software/bc/
[3]: https://en.wikipedia.org/wiki/Rounding#Round_half_away_from_zero
[4]: https://en.wikipedia.org/wiki/Unit_in_the_last_place
[5]: #transcendental-functions
[6]: https://people.eecs.berkeley.edu/~wkahan/LOG10HAF.TXT
